/**HEADER********************************************************************
* 
* Copyright (c) 2010 Avnet Japan Co.,Ltd (NIKE Borad Project.)
* All Rights Reserved
*
*************************************************************************** 
*
* $FileName: usb_device_disk.c$
* $Version : 3.0.6.0$
* $Date    : Aug-4-2009$
*
* Comments:
*
*   Mass storage device demo
*
*END************************************************************************/

#include <usb_dev_types.h>
#include <usb_dev.h>
#include "usb_device_disk.h"
#include <string.h>

#include <descriptors.h>
#include <P5Q_spi.h>

#define  APP_CONTROL_FS_MAX_PKT_SIZE      (32)  /* MCF5275 supports Ep0 32 bytes */
#define  APP_CONTROL_HS_MAX_PKT_SIZE      (64)  /* HS requires bMaxPacketSize0 == 64 */
#define  DEV_DESC_MAX_PACKET_SIZE         (7)
#define  USB_NUM_IFACES                   (1)
#define  EP1_FS_MAX_PACKET_SIZE           (64)
#define  EP1_HS_MAX_PACKET_SIZE           (512)
#define  EP2_FS_MAX_PACKET_SIZE           (EP1_FS_MAX_PACKET_SIZE)
#define  EP2_HS_MAX_PACKET_SIZE           (EP1_HS_MAX_PACKET_SIZE)
#define  CFG_DESC_EP1TX_MAX_PACKET_SIZE   (22)
#define  CFG_DESC_EP1RX_MAX_PACKET_SIZE   (29)

#define  TOTAL_LOGICAL_ADDRESS_BLOCKS     (14335)    //(720)
#define  LENGTH_OF_EACH_LAB               (512)

uchar ep1_buf[BUFFERSIZE];

volatile boolean  ENTER_TEST_MODE = FALSE;
volatile uint_16  test_mode_index = 0;
volatile uchar    speed = 0;

//extern void Main_Task(uint_32 param);
void USB_dev_Task(uint_32 param);

#define MAIN_TASK       10

#if 0
TASK_TEMPLATE_STRUCT  MQX_template_list[] =
{
   { MAIN_TASK,      Main_Task,      2*3000L,  7L, "Main",      MQX_AUTO_START_TASK},
   { 0L,             0L,             0L,   0L, 0L,          0L }
};
#endif

/* Add:1 Dir: IN */
#define BULKIN    ((1 & USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_MASK) << \
                     USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_SHIFT)
/* Add:2 Dir: OUT */
#define BULKOUT   ((2 & USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_MASK) << \
                     USB_DESC_EP_ENDPOINT_ADDRESS_EP_NUMBER_SHIFT)

uint_32 DescriptorSize = USB_DESC_LEN_DEV  +
                           USB_DESC_LEN_CFG  +
                           USB_DESC_LEN_IF   +
                           (2 * USB_DESC_LEN_EP) +
                           USB_DESC_LEN_DEV_QUALIFIER +
                           USB_DESC_LEN_OTHER_SPEED_CFG;

uint_32 DescriptorSizeHS = USB_DESC_LEN_DEV  +
                           USB_DESC_LEN_CFG  +
                           USB_DESC_LEN_IF   +
                           (2 * USB_DESC_LEN_EP) +
                           USB_DESC_LEN_DEV_QUALIFIER +
                           USB_DESC_LEN_OTHER_SPEED_CFG;

uint_8 Descriptor[] =
{
	 /* Device Descriptor */
	   /* Length of DevDesc */
	   USB_DESC_LEN_DEV,
	   /* "Device" Type of descriptor */
	   USB_DESC_TYPE_DEV,
	   /* BCD USB version */
   	USB_uint_16_low(0x0200), USB_uint_16_high(0x0200),
	   /* Device Class is indicated in the interface descriptors */
	   0x00,
	   /* Device Subclass is indicated in the interface descriptors */
	   0x00,
	   /* Mass storage devices do not use class-specific protocols */
	   0x00,
	   /* Max packet size */
	   APP_CONTROL_FS_MAX_PKT_SIZE,
	   /* Vendor ID */
	   USB_uint_16_low(0x05A3), USB_uint_16_high(0x05A3),
	   /* Product ID */
	   USB_uint_16_low(0x1), USB_uint_16_high(0x1),
	   /* BCD Device version */
	   USB_uint_16_low(0x0002), USB_uint_16_high(0x0002),
	   /* Manufacturer string index */
	   0x1,
	   /* Product string index */
	   0x2,
	   /* Serial number string index */
	   0x0,
	   /* Number of configurations available */
	   0x1,
   
	/* Configuration Descriptor - always 9 bytes */
	   USB_DESC_LEN_CFG,
	   /* "Configuration" type of descriptor */
	   USB_DESC_TYPE_CFG,
	   /* Total length of the Configuration descriptor */
	   USB_uint_16_low(USB_DESC_LEN_CFG + USB_DESC_LEN_IF + (2 * USB_DESC_LEN_EP)),
	   USB_uint_16_high(USB_DESC_LEN_CFG + USB_DESC_LEN_IF + (2 * USB_DESC_LEN_EP)),
	   /* NumInterfaces */
	   USB_NUM_IFACES,
	   /* Configuration Value */
	   1,
	   /* Configuration Description String Index */
	   2,
	   /* Attributes.  Self-powered. */
	   (USB_DESC_CFG_ATTRIBUTES_D7_POS | USB_DESC_CFG_ATTRIBUTES_SELF_POWERED_POS),
	   /* Current draw from bus */
	   0,
   
	/* Interface Descriptor - always 9 bytes */
	   USB_DESC_LEN_IF,
	   /* "Interface" type of descriptor */
	   USB_DESC_TYPE_IF,
	   /* Number of this interface */
	   0,
	   /* Alternate Setting */
	   0,
	   /* Number of endpoints on this interface */
	   2,
	   /* Interface Class */
	   0x08,
	   /* Interface Subclass: SCSI transparent command set */
	   0x06,
	   /* Interface Protocol: Bulk only protocol */
	   0x50,
	   /* Interface Description String Index */
	   0,

	/* Endpoint 1 (Bulk In Endpoint), Interface 0 Descriptor - always 7 bytes */
	   USB_DESC_LEN_EP,
	   /* "Endpoint" type of descriptor */
	   USB_DESC_TYPE_EP,
	   /*
	   ** Endpoint address.  The low nibble contains the endpoint number and the
	   ** high bit indicates TX(1) or RX(0).
	   */
	   USB_DESC_EP_ENDPOINT_ADDRESS_DIR_IN | BULKIN,
	   /* Attributes.  0=Control 1=Isochronous 2=Bulk 3=Interrupt */
	   USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_BULK,
	   /* Max Packet Size for this endpoint */
	   USB_uint_16_low(EP1_FS_MAX_PACKET_SIZE), 
	   USB_uint_16_high(EP1_FS_MAX_PACKET_SIZE),
	   /* Polling Interval (ms) */
	   0,
	   
	/* Endpoint 2 (Bulk Out Endpoint), Interface 0 Descriptor - always 7 bytes */
	   USB_DESC_LEN_EP,
	   /* "Endpoint" type of descriptor */
	   USB_DESC_TYPE_EP,
	   /*
	   ** Endpoint address.  The low nibble contains the endpoint number and the
	   ** high bit indicates TX(1) or RX(0).
	   */
	   USB_DESC_EP_ENDPOINT_ADDRESS_DIR_OUT | BULKOUT,
	   /* Attributes.  0=Control 1=Isochronous 2=Bulk 3=Interrupt */
	   USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_BULK,
	   /* Max Packet Size for this endpoint */
	   USB_uint_16_low(EP2_FS_MAX_PACKET_SIZE), 
	   USB_uint_16_high(EP2_FS_MAX_PACKET_SIZE),
	   /* Polling Interval (ms) */
	   0,
	   
   /* Device Qualifier Descriptor - always 10 bytes */
      USB_DESC_LEN_DEV_QUALIFIER,   /* bLength */
      USB_DESC_TYPE_DEV_QUALIFIER,  /* bDescriptorType */
      USB_uint_16_low(0x0200),      /* bcdUSB */
      USB_uint_16_high(0x0200),     /* bcdUSB */
      0x00,                         /* bDeviceClass */
      0x00,                         /* bDeviceSubClass */
      0x00,                         /* bDeviceProtocol */
      APP_CONTROL_HS_MAX_PKT_SIZE,  /* bMaxPacketSize0 */
      0x1,                          /* bNumConfigurations */
      0,                            /* bReserved */

   /* Other Speed Configuration Descriptor - always 9 bytes */
      USB_DESC_LEN_OTHER_SPEED_CFG, /* bLength  */
      USB_DESC_TYPE_OTHER_SPEED_CFG,/* bDescriptorType */
      /* wTotalLength */
      USB_uint_16_low(USB_DESC_LEN_CFG + USB_DESC_LEN_IF + (2 * USB_DESC_LEN_EP)),
      USB_uint_16_high(USB_DESC_LEN_CFG + USB_DESC_LEN_IF + (2 * USB_DESC_LEN_EP)),
      USB_NUM_IFACES,               /* bNumInterfaces */
      1,                            /* bConfigurationValue */
      2,                            /* iConfiguration */
      /* bmAttributes */
      (USB_DESC_CFG_ATTRIBUTES_D7_POS | USB_DESC_CFG_ATTRIBUTES_SELF_POWERED_POS),
      0  /* bMaxPower */
};

uint_8 DescriptorHS[] =
{
	 /* Device Descriptor */
	   /* Length of DevDesc */
	   USB_DESC_LEN_DEV,
	   /* "Device" Type of descriptor */
	   USB_DESC_TYPE_DEV,
	   /* BCD USB version */
   	   USB_uint_16_low(0x0200), USB_uint_16_high(0x0200),
	   /* Device Class is indicated in the interface descriptors */
	   0x00,
	   /* Device Subclass is indicated in the interface descriptors */
	   0x00,
	   /* Mass storage devices do not use class-specific protocols */
	   0x00,
	   /* Max packet size */
	   APP_CONTROL_HS_MAX_PKT_SIZE,
	   /* Vendor ID */
	   USB_uint_16_low(0x05A3), USB_uint_16_high(0x05A3),
	   /* Product ID */
	   USB_uint_16_low(0x1), USB_uint_16_high(0x1),
	   /* BCD Device version */
	   USB_uint_16_low(0x0002), USB_uint_16_high(0x0002),
	   /* Manufacturer string index */
	   0x1,
	   /* Product string index */
	   0x2,
	   /* Serial number string index */
	   0x0,
	   /* Number of configurations available */
	   0x1,
   
	/* Configuration Descriptor - always 9 bytes */
	   USB_DESC_LEN_CFG,
	   /* "Configuration" type of descriptor */
	   USB_DESC_TYPE_CFG,
	   /* Total length of the Configuration descriptor */
	   USB_uint_16_low(USB_DESC_LEN_CFG + USB_DESC_LEN_IF + (2 * USB_DESC_LEN_EP)),
	   USB_uint_16_high(USB_DESC_LEN_CFG + USB_DESC_LEN_IF + (2 * USB_DESC_LEN_EP)),
	   /* NumInterfaces */
	   USB_NUM_IFACES,
	   /* Configuration Value */
	   1,
	   /* Configuration Description String Index */
	   2,
	   /* Attributes.  Self-powered. */
	   (USB_DESC_CFG_ATTRIBUTES_D7_POS | USB_DESC_CFG_ATTRIBUTES_SELF_POWERED_POS),
	   /* Current draw from bus */
	   0,
   
	/* Interface Descriptor - always 9 bytes */
	   USB_DESC_LEN_IF,
	   /* "Interface" type of descriptor */
	   USB_DESC_TYPE_IF,
	   /* Number of this interface */
	   0,
	   /* Alternate Setting */
	   0,
	   /* Number of endpoints on this interface */
	   2,
	   /* Interface Class */
	   0x08,
	   /* Interface Subclass: SCSI transparent command set */
	   0x06,
	   /* Interface Protocol: Bulk only protocol */
	   0x50,
	   /* Interface Description String Index */
	   0,

	/* Endpoint 1 (Bulk In Endpoint), Interface 0 Descriptor - always 7 bytes */
	   USB_DESC_LEN_EP,
	   /* "Endpoint" type of descriptor */
	   USB_DESC_TYPE_EP,
	   /*
	   ** Endpoint address.  The low nibble contains the endpoint number and the
	   ** high bit indicates TX(1) or RX(0).
	   */
	   USB_DESC_EP_ENDPOINT_ADDRESS_DIR_IN | BULKIN,
	   /* Attributes.  0=Control 1=Isochronous 2=Bulk 3=Interrupt */
	   USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_BULK,
	   /* Max Packet Size for this endpoint */
	   USB_uint_16_low(EP1_HS_MAX_PACKET_SIZE),
	   USB_uint_16_high(EP1_HS_MAX_PACKET_SIZE),
	   /* Polling Interval (ms) */
	   0,
	   
	/* Endpoint 2 (Bulk Out Endpoint), Interface 0 Descriptor - always 7 bytes */
	   USB_DESC_LEN_EP,
	   /* "Endpoint" type of descriptor */
	   USB_DESC_TYPE_EP,
	   /*
	   ** Endpoint address.  The low nibble contains the endpoint number and the
	   ** high bit indicates TX(1) or RX(0).
	   */
	   USB_DESC_EP_ENDPOINT_ADDRESS_DIR_OUT | BULKOUT,
	   /* Attributes.  0=Control 1=Isochronous 2=Bulk 3=Interrupt */
	   USB_DESC_EP_ATTRIBUTES_TRANSFER_TYPE_BULK,
	   /* Max Packet Size for this endpoint */
	   USB_uint_16_low(EP2_HS_MAX_PACKET_SIZE),
	   USB_uint_16_high(EP2_HS_MAX_PACKET_SIZE),
	   /* Polling Interval (ms) */
	   0,

   /* Device Qualifier Descriptor - always 10 bytes */
      USB_DESC_LEN_DEV_QUALIFIER,   /* bLength */
      USB_DESC_TYPE_DEV_QUALIFIER,  /* bDescriptorType */
      USB_uint_16_low(0x0200),      /* bcdUSB */
      USB_uint_16_high(0x0200),     /* bcdUSB */
      0x00,                         /* bDeviceClass */
      0x00,                         /* bDeviceSubClass */
      0x00,                         /* bDeviceProtocol */
      APP_CONTROL_FS_MAX_PKT_SIZE,  /* bMaxPacketSize0 */
      0x1,                          /* bNumConfigurations */
      0,                            /* bReserved */

   /* Other Speed Configuration Descriptor - always 9 bytes */
      USB_DESC_LEN_OTHER_SPEED_CFG, /* bLength  */
      USB_DESC_TYPE_OTHER_SPEED_CFG,/* bDescriptorType */
      /* wTotalLength */
      USB_uint_16_low(USB_DESC_LEN_CFG + USB_DESC_LEN_IF + (2 * USB_DESC_LEN_EP)),
      USB_uint_16_high(USB_DESC_LEN_CFG + USB_DESC_LEN_IF + (2 * USB_DESC_LEN_EP)),
      USB_NUM_IFACES,               /* bNumInterfaces */
      1,                            /* bConfigurationValue */
      2,                            /* iConfiguration */
      /* bmAttributes */
      (USB_DESC_CFG_ATTRIBUTES_D7_POS | USB_DESC_CFG_ATTRIBUTES_SELF_POWERED_POS),
      0  /* bMaxPower */
};

uchar USB_IF_ALT[4] = { 0, 0, 0, 0};

/* number of strings in the table not including 0 or n. */
const uint_8 USB_STR_NUM = 3;

/*
** if the number of strings changes, look for USB_STR_0 everywhere and make 
** the obvious changes.  It should be found in 3 places.
*/
uint_8 USB_STR_0[ 4] =
{
   sizeof (USB_STR_0),
	USB_DESC_TYPE_STR,
	USB_uint_16_low (0x0409), USB_uint_16_high (0x0409)
};

uint_8 USB_STR_1[26] =
{
   sizeof (USB_STR_1),
	USB_DESC_TYPE_STR,
	USB_uint_16_low ('M'), USB_uint_16_high ('M'),
 	USB_uint_16_low ('Q'), USB_uint_16_high ('Q'),
  	USB_uint_16_low ('X'), USB_uint_16_high ('X'),
 	USB_uint_16_low (' '), USB_uint_16_high (' '),
 	USB_uint_16_low ('E'), USB_uint_16_high ('E'), 			  			   			  			 
	USB_uint_16_low ('m'), USB_uint_16_high ('m'), 			  			 
	USB_uint_16_low ('b'), USB_uint_16_high ('b'),
	USB_uint_16_low ('e'), USB_uint_16_high ('e'),
	USB_uint_16_low ('d'), USB_uint_16_high ('d'),
	USB_uint_16_low ('d'), USB_uint_16_high ('d'),
	USB_uint_16_low ('e'), USB_uint_16_high ('e'),
	USB_uint_16_low ('d'), USB_uint_16_high ('d')
};

uint_8 USB_STR_2[48] =
{
   sizeof (USB_STR_2),
	USB_DESC_TYPE_STR,
 	USB_uint_16_low ('M'), USB_uint_16_high ('M'),
 	USB_uint_16_low ('Q'), USB_uint_16_high ('Q'),
  	USB_uint_16_low ('X'), USB_uint_16_high ('X'),
 	USB_uint_16_low (' '), USB_uint_16_high (' '),
 	USB_uint_16_low ('M'), USB_uint_16_high ('M'), 			  			   			  			 
	USB_uint_16_low ('a'), USB_uint_16_high ('a'), 			  			 
	USB_uint_16_low ('s'), USB_uint_16_high ('s'),
	USB_uint_16_low ('s'), USB_uint_16_high ('s'),
	USB_uint_16_low (' '), USB_uint_16_high (' '),
	USB_uint_16_low ('S'), USB_uint_16_high ('S'),
	USB_uint_16_low ('t'), USB_uint_16_high ('t'),
	USB_uint_16_low ('o'), USB_uint_16_high ('o'),
	USB_uint_16_low ('r'), USB_uint_16_high ('r'),
	USB_uint_16_low ('a'), USB_uint_16_high ('a'),
	USB_uint_16_low ('g'), USB_uint_16_high ('g'),
 	USB_uint_16_low ('e'), USB_uint_16_high ('e'),
 	USB_uint_16_low (' '), USB_uint_16_high (' '),						 
 	USB_uint_16_low ('D'), USB_uint_16_high ('D'),
 	USB_uint_16_low ('e'), USB_uint_16_high ('e'),
 	USB_uint_16_low ('v'), USB_uint_16_high ('v'),
 	USB_uint_16_low ('i'), USB_uint_16_high ('i'),
 	USB_uint_16_low ('c'), USB_uint_16_high ('c'),
 	USB_uint_16_low ('e'), USB_uint_16_high ('e')
};

const uint_8_ptr USB_STRING_DESCS[USB_STR_NUM] =
{
   (uint_8_ptr)((pointer)USB_STR_0),
   (uint_8_ptr)((pointer)USB_STR_1),
   (uint_8_ptr)((pointer)USB_STR_2)
};

boolean        CBW_PROCESSED = FALSE;
CSW_STRUCT     csw;

MASS_STORAGE_DEVICE_INFO_STRUCT device_information_data = {
   0, 0x80, 0, 0x01, 0x1F, 0, 0, 0, 
   /* Vendor information: "MQXEmbed" */
   {'M', 'Q', 'X', 'E', 'm', 'b', 'e','d'}, 
   /* Product information: "Disk            " */
   {'D', 'i', 's', 'k', ' ', ' ', ' ', ' ',
   ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '}, 
   /* Product Revision level: "Demo" */
   {'D', 'e', 'm', 'o'}
}; 

MASS_STORAGE_READ_CAPACITY_STRUCT read_capacity = {
   /* Data for the capacity */
   {
      0x00, 0x00, USB_uint_16_high(TOTAL_LOGICAL_ADDRESS_BLOCKS - 14), 
      USB_uint_16_low(TOTAL_LOGICAL_ADDRESS_BLOCKS - 14)
   }, 
   {
      0x00, 0x00, USB_uint_16_high(LENGTH_OF_EACH_LAB), 
      USB_uint_16_low(LENGTH_OF_EACH_LAB)
   }
};

uchar_ptr MASS_STORAGE_DISK;

uchar BOOT_SECTOR_AREA[512] = {
   /* Block 0 is the boot sector. Following is the data in the boot sector */
   /* 80x86 "short: jump instruction, indicating that the disk is formatted */
    0xEB, 
    /* 8-bit displacement */
    0x3C, 
    /* NOP OPCode */
    0x90, 
    /* 8-bytes for OEM identification: "ARC 4.3 " */
    'A', 'R', 'C', ' ', '4', '.', '3', ' ', 
    /* bytes/sector: 512 bytes (0x0200) */
    0x00, 0x02, 
    /* Sectors/allocation unit */
    0x01,
    /* Reserved sectors: 0x0001 */
    0x01, 0x00, 
    /* Number of File Allocation Tables (FATs): 2 */
    0x02,
    /* Number of root directory entries */
    0x70, 0x00, 
    /* Total sectors in logical volume */
    USB_uint_16_low(TOTAL_LOGICAL_ADDRESS_BLOCKS), 
    USB_uint_16_high(TOTAL_LOGICAL_ADDRESS_BLOCKS),
    /* Media descriptor byte: 0xF8: Fixed disk */
    0xFD,
    /* Sectors/FAT: 3 (Each FAT starts at a new sector) */
    0x03, 0x00, 
    /* Sectors/track: 9 */
    0x09, 0x00, 
    /* Number of heads */
    0x02, 0x00, 
    /* Number of hidden sectors: 0 */
    0x00, 0x00, 0x00, 0x00, 
    /* Total sectors in logical volume */
    0x00, 0x00, 0x00, 0x00, 
    /* Physical drive number */
    0x00, 
    /* Reserved */
    0x00, 
    /* Extended boot signature record: 0x29 */
    0x29,
    /* 32-bit binary volume ID */
    0x01, 0x02, 0x03, 0x04, 
    /* Volume label */
    'M', 'Q', 'X', 'E', 'm', 'b', 'e', 'd', 'd', 'e', 'd', 
    /* Reserved */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    /* Bootstrap */
    0x33, 0xC0, 0x8E, 0xD0, 0xBC, 0x00, 0x7C, 0xFC, 0xE8, 0x45, 0x00, 
    /* String: \r\nNon-System disk\r\nPress any key to reboot\r\n" */
    0x0D, 0x0A, 0x4E, 0x6F, 0x6E, 0x2D, 0x53, 0x79, 0x73, 0x74, 0x65,
    0x6D, 0x20, 0x64, 0x69, 0x73, 0x6B, 0x0D, 0x0A, 0x50, 0x72, 0x65, 
    0x73, 0x73, 0x20, 0x61, 0x6E, 0x79, 0x20, 0x6B, 0x65, 0x79, 0x20, 
    0x74, 0x6F, 0x20, 0x72, 0x65, 0x62, 0x6F, 0x6F, 0x74, 0x0D, 0x0A, 
    0x5E, 0xEB, 0x02, 0xCD, 0x10, 0xB4, 0x0E, 0xBB, 0x07, 0x00, 0x2E, 
    0xAC, 0x84, 0xC0, 0x75, 0xF3, 0x98, 0xCD, 0x16, 0xCD, 0x19, 0xEB, 
    0xB1, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    /* Partition descriptors */
    0x00, 0x01, 0x01, 0x00, 0x01, 0x04, 0x02, 0xC8, 0x11, 0x00, 0x00, 
    0x00, 0x02, 0xD0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x55, 0xAA
};

uchar FAT16_SPECIAL_BYTES[3] = {
   /* FAT ID: Same as Media descriptor */
   0xFD, 0xFF, 0xFF
};

uint_16        usb_status;
uint_8         endpoint, if_status;
uint_8         data_to_send;
uint_16        sof_count;
uint_8         setup_packet[sizeof(SETUP_STRUCT)];
SETUP_STRUCT   local_setup_packet;

#define PrintUnhandledSetup(text) printf("Setup not handled: %02X %02X %04X %04X %04X" text "\n", \
									local_setup_packet.REQUESTTYPE,\
									local_setup_packet.REQUEST,\
									local_setup_packet.VALUE,\
									local_setup_packet.INDEX,\
									local_setup_packet.LENGTH);

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : ch9ClearFeature
* Returned Value : None
* Comments       :
*     Chapter 9 ClearFeature command
* 
*END*--------------------------------------------------------------------*/
static void ch9ClearFeature
   (
      /* USB handle */
      _usb_device_handle handle,
      
      /* Is it a Setup phase? */
      boolean setup,
            
      /* The setup packet pointer */
      SETUP_STRUCT_PTR setup_ptr
   )
{ /* Body */
   static uint_8           endpoint;
   uint_16                 usb_status;

   _usb_device_get_status(handle, USB_STATUS_DEVICE_STATE, &usb_status);

   if ((usb_status != USB_STATE_CONFIG) && (usb_status != USB_STATE_ADDRESS)) {
      _usb_device_stall_endpoint(handle, 0, 0);
      return;
   } /* Endif */

   if (setup) {
      switch (setup_ptr->REQUESTTYPE) {
      
         case 0:
            /* DEVICE */
            if (setup_ptr->VALUE == 1) {
               /* clear remote wakeup */
               _usb_device_get_status(handle, USB_STATUS_DEVICE, &usb_status);
               usb_status &= (uint_16)~USB_REMOTE_WAKEUP;
               _usb_device_set_status(handle, USB_STATUS_DEVICE, usb_status);
            } else {
               _usb_device_stall_endpoint(handle, 0, 0);
               return;
            } /* Endif */
            break;
         
         case 2:
            /* ENDPOINT */
            if (setup_ptr->VALUE != 0) {
               _usb_device_stall_endpoint(handle, 0, 0);
               return;
            } /* Endif */
            endpoint = (uint_8)(setup_ptr->INDEX & USB_STATUS_ENDPOINT_NUMBER_MASK);
            _usb_device_get_status(handle, (uint_8)(USB_STATUS_ENDPOINT | endpoint),
               &usb_status);
            /* unstall */
            _usb_device_set_status(handle, (uint_8)(USB_STATUS_ENDPOINT | endpoint),
               0);
            break;
         default:
            _usb_device_stall_endpoint(handle, 0, 0);
            return;
      } /* Endswitch */
      
      /* status phase */
      _usb_device_send_data(handle, 0, 0, 0);
   } /* Endif */
   return;
} /* Endbody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : ch9SetFeature
* Returned Value : None
* Comments       :
*     Chapter 9 SetFeature command
* 
*END*--------------------------------------------------------------------*/
static void ch9SetFeature
   (
      /* USB handle */
      _usb_device_handle handle,
      
      /* Is it a Setup phase? */
      boolean setup,
            
      /* The setup packet pointer */
      SETUP_STRUCT_PTR setup_ptr
   )
{ /* Body */
   uint_16           usb_status;
   uint_8            endpoint;

   if (setup) {
      switch (setup_ptr->REQUESTTYPE) {

         case 0:
            /* DEVICE */
            switch (setup_ptr->VALUE) {
               case 1:
                  /* set remote wakeup */
                  _usb_device_get_status(handle, USB_STATUS_DEVICE, &usb_status);
                  usb_status |= USB_REMOTE_WAKEUP;
                  _usb_device_set_status(handle, USB_STATUS_DEVICE, usb_status);
                  break;
               case 2:
                  /* Test Mode */
                  if ((setup_ptr->INDEX & 0x00FF) || 
                     (speed != USB_SPEED_HIGH)) 
                  {
                     _usb_device_stall_endpoint(handle, 0, 0);
                     return;
                  } /* Endif */
                  _usb_device_get_status(handle, USB_STATUS_DEVICE_STATE, 
                     &usb_status);
                  if ((usb_status == USB_STATE_CONFIG) || 
                     (usb_status == USB_STATE_ADDRESS) || 
                     (usb_status == USB_STATE_DEFAULT)) 
                  {
                     ENTER_TEST_MODE = TRUE;
                     test_mode_index = (uint_16)(setup_ptr->INDEX & 0xFF00);
                  } else {
                     _usb_device_stall_endpoint(handle, 0, 0);
                     return;
                  } /* Endif */
                  break;
               default:
                  _usb_device_stall_endpoint(handle, 0, 0);
                  return;
            } /* Endswitch */
            break;
            
         case 2:
            /* ENDPOINT */
            if (setup_ptr->VALUE != 0) {
               _usb_device_stall_endpoint(handle, 0, 0);
               return;
            } /* Endif */
            endpoint = (uint_8)(setup_ptr->INDEX & USB_STATUS_ENDPOINT_NUMBER_MASK);
            _usb_device_get_status(handle, (uint_8)(USB_STATUS_ENDPOINT | endpoint),
               &usb_status);
            /* set stall */
            _usb_device_set_status(handle, (uint_8)(USB_STATUS_ENDPOINT | endpoint),
               1);
            break;

         default:
            _usb_device_stall_endpoint(handle, 0, 0);
            return;
      } /* Endswitch */
      
      /* status phase */
      _usb_device_send_data(handle, 0, 0, 0);
   } else {
      if (ENTER_TEST_MODE) {
         /* Enter Test Mode */
         _usb_device_set_status(handle, USB_STATUS_TEST_MODE, 
            test_mode_index);
      } /* Endif */
   } /* Endif */
   return;
} /* Endbody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : ch9GetDescriptor
* Returned Value : None
* Comments       :
*     Chapter 9 GetDescriptor command
*     The Device Request can ask for Device/Config/string/interface/endpoint
*     descriptors (via wValue). We then post an IN response to return the
*     requested descriptor.
*     And then wait for the OUT which terminates the control transfer.
* 
*END*--------------------------------------------------------------------*/
static void ch9GetDescriptor
   (
      /* USB handle */
      _usb_device_handle handle,
      
      /* Is it a Setup phase? */
      boolean setup,
            
      /* The setup packet pointer */
      SETUP_STRUCT_PTR setup_ptr
   )
{ /* Body */
   uint_16 wvalue;
   uint_8  wvalue_desc_type;
   uint_8  wvalue_desc_index;

   wvalue = setup_ptr->VALUE;
   wvalue_desc_type = (uint_8)((wvalue & (0xFF << 8)) >> 8);
   wvalue_desc_index = (uint_8)((wvalue & (0xFF << 0)) >> 0);

   if (setup) {
      /* Load the appropriate string depending on the descriptor requested.*/
      switch (wvalue_desc_type) {

         case USB_DESC_TYPE_DEV:
         	/* DEVICE descriptor is handled by hardware */
         	PrintUnhandledSetup();
            break;

         case USB_DESC_TYPE_CFG:
        	/* CONFIGURATION descriptor is handled by hardware */
            PrintUnhandledSetup();
            break;

         case USB_DESC_TYPE_STR:
            if ((setup_ptr->VALUE & 0x00FF) > USB_STR_NUM) {
               _usb_device_send_data(handle, 0, USB_STRING_DESCS[USB_STR_NUM+1],
                  (uint_32)MIN(setup_ptr->LENGTH, USB_STRING_DESCS[USB_STR_NUM+1][0]));
            } else {
               _usb_device_send_data(handle, 0,
                  USB_STRING_DESCS[setup_ptr->VALUE & 0x00FF],
                  (uint_32)MIN(setup_ptr->LENGTH,
                     USB_STRING_DESCS[setup_ptr->VALUE & 0x00FF][0]));
            } /* Endif */      
            break;
            
         case USB_DESC_TYPE_DEV_QUALIFIER:
         	/* DEVICE QUALIFIER descriptor is handled by hardware */
            PrintUnhandledSetup();
            break;
            
         case USB_DESC_TYPE_OTHER_SPEED_CFG:
         	/* OTHER SPEED CONFIGURATION descriptor is handled by hardware */
            PrintUnhandledSetup();
            break;

         default:
            _usb_device_stall_endpoint(handle, 0, 0);
            return;
      } /* Endswitch */
      /* status phase */
      
      _usb_device_recv_data(handle, 0, 0, 8);   

   } /* Endif */
   return;
} /* Endbody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : ch9SetDescriptor
* Returned Value : None
* Comments       :
*     Chapter 9 SetDescriptor command
* 
*END*--------------------------------------------------------------------*/
static void ch9SetDescriptor
   (
      /* USB handle */
      _usb_device_handle handle,
      
      /* Is it a Setup phase? */
      boolean setup,
            
      /* The setup packet pointer */
      SETUP_STRUCT_PTR setup_ptr
   )
{ /* Body */
   _usb_device_stall_endpoint(handle, 0, 0);
   return;
} /* Endbody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : ch9SetConfig
* Returned Value : None
* Comments       :
*     Chapter 9 SetConfig command
* 
*END*--------------------------------------------------------------------*/
static void ch9SetConfig
   (
      /* USB handle */
      _usb_device_handle handle,
      
      /* Is it a Setup phase? */
      boolean setup,
            
      /* The setup packet pointer */
      SETUP_STRUCT_PTR setup_ptr
   )
{ /* Body */
   uint_16 usb_state;
   
   if (setup) {
      if ((setup_ptr->VALUE & 0x00FF) > 1) {
         /* generate stall */
         _usb_device_stall_endpoint(handle, 0, 0);
         return;
      } /* Endif */
      /* 0 indicates return to unconfigured state */
      if ((setup_ptr->VALUE & 0x00FF) == 0) {
         _usb_device_get_status(handle, USB_STATUS_DEVICE_STATE, &usb_state);
         if ((usb_state == USB_STATE_CONFIG) || 
            (usb_state == USB_STATE_ADDRESS)) 
         {
            /* clear the currently selected config value */
            _usb_device_set_status(handle, USB_STATUS_CURRENT_CONFIG, 0);
            _usb_device_set_status(handle, USB_STATUS_DEVICE_STATE,
               USB_STATE_ADDRESS);
            /* status phase */      
            _usb_device_send_data(handle, 0, 0, 0);
            
            _usb_device_recv_data(handle, 0, 0, 8);   

         } else {
            _usb_device_stall_endpoint(handle, 0, 0);
         } /* Endif */
         return;
      } /* Endif */

      /*
      ** If the configuration value (setup_ptr->VALUE & 0x00FF) differs
      ** from the current configuration value, then endpoints must be
      ** reconfigured to match the new device configuration
      */
      _usb_device_get_status(handle, USB_STATUS_CURRENT_CONFIG,
         &usb_state);
      if (usb_state != (setup_ptr->VALUE & 0x00FF)) {
         /* Reconfigure endpoints here */
         switch (setup_ptr->VALUE & 0x00FF) {
      
            default:
               break;
         } /* Endswitch */
         _usb_device_set_status(handle, USB_STATUS_CURRENT_CONFIG,
            (uint_16)(setup_ptr->VALUE & 0x00FF));
         _usb_device_set_status(handle, USB_STATUS_DEVICE_STATE,
            USB_STATE_CONFIG);      
         /* status phase */      
         _usb_device_send_data(handle, 0, 0, 0);
         
         _usb_device_recv_data(handle, 0, 0, 8);   
         
         return;
      } /* Endif */
      _usb_device_set_status(handle, USB_STATUS_DEVICE_STATE,
         USB_STATE_CONFIG);
      /* status phase */
      _usb_device_send_data(handle, 0, 0, 0);
      
      _usb_device_recv_data(handle, 0, 0, 8);   
      
   } /* Endif */
   return;
} /* Endbody */


/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : ch9SetInterface
* Returned Value : None
* Comments       :
*     Chapter 9 SetInterface command
* 
*END*--------------------------------------------------------------------*/
static void ch9SetInterface
   (
      /* USB handle */
      _usb_device_handle handle,
      
      /* Is it a Setup phase? */
      boolean setup,
            
      /* The setup packet pointer */
      SETUP_STRUCT_PTR setup_ptr
   )
{ /* Body */
   if (setup) {
      if (setup_ptr->REQUESTTYPE != 0x01) {
         _usb_device_stall_endpoint(handle, 0, 0);
         return;
      } /* Endif */

      /*
      ** If the alternate value (setup_ptr->VALUE & 0x00FF) differs
      ** from the current alternate value for the specified interface,
      ** then endpoints must be reconfigured to match the new alternate
      */
      if (USB_IF_ALT[setup_ptr->INDEX & 0x00FF]
         != (setup_ptr->VALUE & 0x00FF))
      {
         USB_IF_ALT[setup_ptr->INDEX & 0x00FF] = (uchar)(setup_ptr->VALUE & 0x00FF);
         /* Reconfigure endpoints here. */
         
      } /* Endif */

      /* status phase */
      _usb_device_send_data(handle, 0, 0, 0);
   } /* Endif */
   return;
} /* Endbody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : ch9SynchFrame
* Returned Value : 
* Comments       :
*     Chapter 9 SynchFrame command
* 
*END*--------------------------------------------------------------------*/
static void ch9SynchFrame
   (
      /* USB handle */
      _usb_device_handle handle,
      
      /* Is it a Setup phase? */
      boolean setup,
            
      /* The setup packet pointer */
      SETUP_STRUCT_PTR setup_ptr
   )
{ /* Body */
   
   if (setup) {
      if (setup_ptr->REQUESTTYPE != 0x02) {
         _usb_device_stall_endpoint(handle, 0, 0);
         return;
      } /* Endif */

	  /* We support only 1 interface */
      if ((setup_ptr->INDEX & 0x00FF) >= USB_NUM_IFACES)
      {
         _usb_device_stall_endpoint(handle, 0, 0);
         return;
      } /* Endif */

      _usb_device_get_status(handle, USB_STATUS_SOF_COUNT, &sof_count);
      _usb_device_send_data(handle, 0, (uchar_ptr)((pointer)&sof_count),
         MIN(setup_ptr->LENGTH, sizeof(sof_count)));
      /* status phase */      
      _usb_device_recv_data(handle, 0, 0, 0);
   } /* Endif */
   return;
} /* Endbody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : ch9Class
* Returned Value : 
* Comments       :
*     Chapter 9 Class specific request
*     See section 9.4.11 (page 195) of the USB 1.1 Specification.
* 
*END*--------------------------------------------------------------------*/
static void ch9Class
   (
      _usb_device_handle handle,
      boolean setup,
      SETUP_STRUCT_PTR setup_ptr
   )
{ /* Body */
    uint_8 ep_num = 0;
    uint_8 lun = 0;
   
    if (setup) {
        switch (setup_ptr->REQUEST) {
        case 0xFF:
            /* Bulk-Only Mass Storage Reset: Ready the device for the next 
            ** CBW from the host 
            */
            
            break;
        case 0xFE:
            /* Get Max LUN */
            _usb_device_send_data(handle, ep_num, &lun, 1);
            _usb_device_recv_data(handle, 0, 0, 8);
            return;
        default:
            _usb_device_stall_endpoint(handle, ep_num, 0);
        } /* EndSwitch */
        
        _usb_device_send_data(handle, 0, 0, 0);
        _usb_device_recv_data(handle, 0, 0, 8);
    } /* Endif */

} /* Endbody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : service_ep0
* Returned Value : None
* Comments       :
*     Called upon a completed endpoint 0 (USB 1.1 Chapter 9) transfer
* 
*END*--------------------------------------------------------------------*/
static void service_ep0
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
      
      /* [IN] Is it a setup packet? */
      boolean              setup,
      
      /* [IN] Direction of the transfer.  Is it transmit? */
      uint_8               direction,
      
      /* [IN] Pointer to the data buffer */
      uint_8_ptr           buffer,
      
      /* [IN] Length of the transfer */
      uint_32              length      
   )
{ /* Body */
   
   if (!setup)
      return;
   
   _usb_device_read_setup_data(handle, 0, (uchar_ptr)&local_setup_packet);
   
   if(local_setup_packet.REQUESTTYPE == 0xA0)
      local_setup_packet.REQUESTTYPE = 0x81;
   
   switch (local_setup_packet.REQUESTTYPE & 0x60) {

      case 0x00:
         switch (local_setup_packet.REQUEST) {

            case 0x0:
            	/* GET_STATUS is handled by hardware */
               PrintUnhandledSetup();
               break;

            case 0x1:
               ch9ClearFeature(handle, setup, &local_setup_packet);
               break;

            case 0x3:
               ch9SetFeature(handle, setup, &local_setup_packet);
               break;

            case 0x5:
               /* SET_ADDRESS is called only in debug mode */
               PrintUnhandledSetup();
               break;

            case 0x6:
               ch9GetDescriptor(handle, setup, &local_setup_packet);
               break;

            case 0x7:
               ch9SetDescriptor(handle, setup, &local_setup_packet);
               break;

            case 0x8:
            	/* GET_CONFIGURATION is handled by hardware */
            	PrintUnhandledSetup();
               break;

            case 0x9:
               ch9SetConfig(handle, setup, &local_setup_packet);
               break;

            case 0xa:
            	/* GET_INTERFACE is handled by hardware */
            	PrintUnhandledSetup();
               break;

            case 0xb:
               ch9SetInterface(handle, setup, &local_setup_packet);
               break;

            case 0xc:
               ch9SynchFrame(handle, setup, &local_setup_packet);
               break;

            default:
               _usb_device_stall_endpoint(handle, 0, 0);
               break;

         } /* Endswitch */
         
         break;

      case 0x20:
         /* class specific request */
         ch9Class(handle,setup,&local_setup_packet);
         break;
         /* end */
         /* Queue your own packet for class data here */

      case 0x40:
         /* vendor specific request */
          _usb_device_send_data(handle, 0, 0, 0);
         break;
      
      default:
         _usb_device_stall_endpoint(handle, 0, 0);
         break;
         
   } /* Endswitch */
   
    
   return;
} /* Endbody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : _process_inquiry_command
* Returned Value : None
* Comments       :
*     Process a Mass storage class Inquiry command
* 
*END*--------------------------------------------------------------------*/
static void _process_inquiry_command
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
            
      /* [IN] Endpoint number */
      uint_8               ep_num,
      
      /* [IN] Pointer to the data buffer */
      CBW_STRUCT_PTR       cbw_ptr
   )
{ /* Body */
   MASS_STORAGE_INQUIRY_PTR inquiry_cmd_ptr = (MASS_STORAGE_INQUIRY_PTR)((pointer)cbw_ptr->CBWCB);
   
   if (cbw_ptr->DCBWDATALENGTH) {
      if (cbw_ptr->BMCBWFLAGS & USB_CBW_DIRECTION_BIT) {      
         /* Send the device information */
         _usb_device_send_data(handle, BULKIN, (uchar_ptr)&device_information_data, 36);
      } /* Endif */
   } /* Endif */
   
   /* The actual length will never exceed the DCBWDATALENGTH */            
   csw.DCSWDATARESIDUE = (cbw_ptr->DCBWDATALENGTH - 36);
   csw.BCSWSTATUS = 0;
   
} /* EndBody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : _process_unsupported_command
* Returned Value : None
* Comments       :
*     Responds appropriately to unsupported commands
* 
*END*--------------------------------------------------------------------*/
static void _process_unsupported_command
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
            
      /* [IN] Endpoint number */
      uint_8               ep_num,
      
      /* [IN] Pointer to the data buffer */
      CBW_STRUCT_PTR       cbw_ptr
   )
{ /* Body */
   /* The actual length will never exceed the DCBWDATALENGTH */
   csw.DCSWDATARESIDUE = 0;
   csw.BCSWSTATUS = 0;

   if (cbw_ptr->BMCBWFLAGS & USB_CBW_DIRECTION_BIT) {      
      /* Send a zero-length packet */
 
      _usb_device_send_data(handle, BULKIN, (uchar_ptr)&device_information_data, 0);
   } else {
      CBW_PROCESSED = FALSE;
      /* Send the command status information */
      _usb_device_send_data(handle, BULKIN, (uchar_ptr)&csw, 13);           
      _usb_device_recv_data(handle, BULKOUT, ep1_buf, 31);
   } /* Endif */
   
} /* EndBody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : _process_report_capacity
* Returned Value : None
* Comments       :
*     Reports the media capacity as a response to READ CAPACITY Command.
* 
*END*--------------------------------------------------------------------*/
static void _process_report_capacity
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
            
      /* [IN] Endpoint number */
      uint_8               ep_num,
      
      /* [IN] Pointer to the data buffer */
      CBW_STRUCT_PTR       cbw_ptr
   )
{ /* Body */
/*   uchar szTmp[8] = {
      0x00, 0x00, USB_uint_16_high(TOTAL_LOGICAL_ADDRESS_BLOCKS-14), 
      USB_uint_16_low(TOTAL_LOGICAL_ADDRESS_BLOCKS-14),
      0x00, 0x00, USB_uint_16_high(LENGTH_OF_EACH_LAB), 
      USB_uint_16_low(LENGTH_OF_EACH_LAB)
   }; */
   
   if (cbw_ptr->BMCBWFLAGS & USB_CBW_DIRECTION_BIT) {      
      /* Send a zero-length packet */
     _usb_device_send_data(handle, BULKIN, (uchar_ptr)&read_capacity, 8);
   } /* Endif */
   
   /* The actual length will never exceed the DCBWDATALENGTH */            
   csw.DCSWDATARESIDUE = 0;
   csw.BCSWSTATUS = 0;
   
} /* EndBody */

extern  FILE_PTR                   pcm_dev_handle1;

uchar mass_storage_disk_buffer[8*512];
PCM_SPI_MAMORY_IO  pcm_data_mem;

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : _process_read_command
* Returned Value : None
* Comments       :
*     Sends data as a response to READ Command.
* 
*END*--------------------------------------------------------------------*/
static void _process_read_command
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
            
      /* [IN] Endpoint number */
      uint_8               ep_num,
      
      /* [IN] Pointer to the data buffer */
      CBW_STRUCT_PTR       cbw_ptr
   )
{ /* Body */
   uint_32 index1 = 0, index2 = 0;

   if (cbw_ptr->BMCBWFLAGS & USB_CBW_DIRECTION_BIT) {      
      /* Send a zero-length packet */
      index1  = ((uint_32)cbw_ptr->CBWCB[4] << 8);
      index1  |= cbw_ptr->CBWCB[5];
      index2 = ((uint_32)cbw_ptr->CBWCB[7] << 8);
      index2 |= (uint_32)cbw_ptr->CBWCB[8];
//      _usb_device_send_data(handle, BULKIN, MASS_STORAGE_DISK + (index1*LENGTH_OF_EACH_LAB), LENGTH_OF_EACH_LAB * index2);

		printf("process_read_command [%d]%d \n", (index1*LENGTH_OF_EACH_LAB), LENGTH_OF_EACH_LAB * index2);
#if 0
		pcm_data_mem.addr = (index1*LENGTH_OF_EACH_LAB);
		pcm_data_mem.data = &mass_storage_disk_buffer[0];
		pcm_data_mem.size = (LENGTH_OF_EACH_LAB * index2);
		ioctl(pcm_dev_handle1, IO_IOCTL_QSPI_BLOCK_GET_MEM, (pointer)&pcm_data_mem);
		_usb_device_send_data(handle, BULKIN, mass_storage_disk_buffer, (LENGTH_OF_EACH_LAB * index2));
#endif

#if 0
		if ((LENGTH_OF_EACH_LAB * index2)>512){
			pcm_data_mem.addr = (index1*LENGTH_OF_EACH_LAB);
			pcm_data_mem.data = &mass_storage_disk_buffer[0];
			pcm_data_mem.size = 512;
			ioctl(pcm_dev_handle1, IO_IOCTL_QSPI_BLOCK_GET_MEM, (pointer)&pcm_data_mem);
			_usb_device_send_data(handle, BULKIN, mass_storage_disk_buffer, 512);
		}
		else{
			pcm_data_mem.addr = (index1*LENGTH_OF_EACH_LAB);
			pcm_data_mem.data = &mass_storage_disk_buffer[0];
			pcm_data_mem.size = (LENGTH_OF_EACH_LAB * index2);
			ioctl(pcm_dev_handle1, IO_IOCTL_QSPI_BLOCK_GET_MEM, (pointer)&pcm_data_mem);
			_usb_device_send_data(handle, BULKIN, mass_storage_disk_buffer, (LENGTH_OF_EACH_LAB * index2));
		}
#endif
#if 1
		{
			int_32  size;
			int_32  addr;
			int_32  length;
			uint_8  ercd;

			length = (LENGTH_OF_EACH_LAB * index2);
			addr = (index1*LENGTH_OF_EACH_LAB);

			while(length>0){

				if (length>512){
					size = 512;
				}
				else{
					size = length;
				}

      			pcm_data_mem.addr = addr;
      			pcm_data_mem.data = &mass_storage_disk_buffer[0];
      			pcm_data_mem.size = size;
      			ioctl(pcm_dev_handle1, IO_IOCTL_QSPI_BLOCK_GET_MEM, (pointer)&pcm_data_mem);

      			ercd = _usb_device_send_data(handle, BULKIN, mass_storage_disk_buffer, size);
				printf("process_send %d \n",ercd);

				length = length - size;
				addr = addr + size;
			}
		}
#endif

   } /* Endif */

   /* The actual length will never exceed the DCBWDATALENGTH */            
   csw.DCSWDATARESIDUE = (cbw_ptr->DCBWDATALENGTH - LENGTH_OF_EACH_LAB*index2);
   csw.BCSWSTATUS = 0;
   
} /* EndBody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : _process_write_command
* Returned Value : None
* Comments       :
*     Sends data as a response to WRITE Command.
* 
*END*--------------------------------------------------------------------*/
static void _process_write_command
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
            
      /* [IN] Endpoint number */
      uint_8               ep_num,
      
      /* [IN] Pointer to the data buffer */
      CBW_STRUCT_PTR       cbw_ptr
   )
{ /* Body */
   uint_32 index1 = 0, index2 = 0;

   if (!(cbw_ptr->BMCBWFLAGS & USB_CBW_DIRECTION_BIT)) {
      /* Send a zero-length packet */
      index1  = ((uint_32)cbw_ptr->CBWCB[4] << 8);
      index1  |= cbw_ptr->CBWCB[5];
      index2 = ((uint_32)cbw_ptr->CBWCB[7] << 8);
      index2 |= (uint_32)cbw_ptr->CBWCB[8];
      if (_usb_device_get_transfer_status(handle, ep_num, USB_RECV) != USB_STATUS_IDLE) {
         _usb_device_cancel_transfer(handle, ep_num, USB_RECV);
      } /* Endif */

//      _usb_device_recv_data(handle, ep_num, MASS_STORAGE_DISK + (index1*LENGTH_OF_EACH_LAB), LENGTH_OF_EACH_LAB * index2);

      _usb_device_recv_data(handle, ep_num, mass_storage_disk_buffer, LENGTH_OF_EACH_LAB * index2);

	  printf("process_write_command %d \n",LENGTH_OF_EACH_LAB * index2);

      pcm_data_mem.addr = (index1*LENGTH_OF_EACH_LAB);
      pcm_data_mem.data = &mass_storage_disk_buffer[0];
      pcm_data_mem.size = LENGTH_OF_EACH_LAB * index2;
      ioctl(pcm_dev_handle1, IO_IOCTL_QSPI_BLOCK_SET_MEM, (pointer)&pcm_data_mem);

   } /* Endif */

   /* The actual length will never exceed the DCBWDATALENGTH */            
   csw.DCSWDATARESIDUE = (cbw_ptr->DCBWDATALENGTH - LENGTH_OF_EACH_LAB*index2);
   csw.BCSWSTATUS = 0;
   
} /* EndBody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : _process_test_unit_ready
* Returned Value : None
* Comments       :
*     Responds appropriately to unit ready query
* 
*END*--------------------------------------------------------------------*/
static void _process_test_unit_ready
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
            
      /* [IN] Endpoint number */
      uint_8               ep_num,
      
      /* [IN] Pointer to the data buffer */
      CBW_STRUCT_PTR       cbw_ptr
   )
{ /* Body */

   /* The actual length will never exceed the DCBWDATALENGTH */
   csw.DCSWDATARESIDUE = 0;
   csw.BCSWSTATUS = 0;
   
   CBW_PROCESSED = FALSE;
   /* Send the command status information */
   _usb_device_send_data(handle, BULKIN, (uchar_ptr)&csw, 13);           
   _usb_device_recv_data(handle, BULKOUT, ep1_buf, 31);  
   
} /* EndBody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : _process_prevent_allow_medium_removal
* Returned Value : None
* Comments       :
*     Responds appropriately to unit ready query
* 
*END*--------------------------------------------------------------------*/
static void _process_prevent_allow_medium_removal
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
            
      /* [IN] Endpoint number */
      uint_8               ep_num,
      
      /* [IN] Pointer to the data buffer */
      CBW_STRUCT_PTR       cbw_ptr
   )
{ /* Body */

   /* The actual length will never exceed the DCBWDATALENGTH */
   csw.DCSWDATARESIDUE = 0;
   csw.BCSWSTATUS = 0;
   
   CBW_PROCESSED = FALSE;
   /* Send the command status information */
   _usb_device_send_data(handle, BULKIN, (uchar_ptr)&csw, 13);         
   _usb_device_recv_data(handle, BULKOUT, ep1_buf, 31);
   
} /* EndBody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : _process_mass_storage_command
* Returned Value : None
* Comments       :
*     Process a Mass storage class command
* 
*END*--------------------------------------------------------------------*/
static void _process_mass_storage_command
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
            
      /* [IN] Endpoint number */
      uint_8               ep_num,
      
      /* [IN] Pointer to the data buffer */
      CBW_STRUCT_PTR       cbw_ptr
   )
{ /* Body */
   switch (cbw_ptr->CBWCB[0]) {
      case 0x00: /* Request the device to report if it is ready */
         _process_test_unit_ready(handle, ep_num, cbw_ptr);
         break;
      case 0x12: /* Inquity command. Get device information */
         _process_inquiry_command(handle, ep_num, cbw_ptr);
         break;
      case 0x1A:
         _process_unsupported_command(handle, ep_num, cbw_ptr);
         break;
      case 0x1E: /* Prevent or allow the removal of media from a removable 
                 ** media device 
                 */
         _process_prevent_allow_medium_removal(handle, ep_num, cbw_ptr);
         break;
      case 0x23: /* Read Format Capacities. Report current media capacity and 
                 ** formattable capacities supported by media 
                 */
         /* We bahave like already installed medium. No need to send any data */
         _process_unsupported_command(handle, ep_num, cbw_ptr);
         break;
      case 0x25: /* Report current media capacity */
         _process_report_capacity(handle, ep_num, cbw_ptr);
         break;
      case 0x28: /* Read (10) Transfer binary data from media to the host */
         _process_read_command(handle, ep_num, cbw_ptr);
         break;
      case 0x2A: /* Write (10) Transfer binary data from the host to the 
                 ** media 
                 */
         _process_write_command(handle, 2, cbw_ptr);
         break;
      case 0x01: /* Position a head of the drive to zero track */
      case 0x03: /* Transfer status sense data to the host */
      case 0x04: /* Format unformatted media */
      case 0x1B: /* Request a request a removable-media device to load or 
                 ** unload its media 
                 */
      case 0x1D: /* Perform a hard reset and execute diagnostics */
      case 0x2B: /* Seek the device to a specified address */
      case 0x2E: /* Transfer binary data from the host to the media and 
                 ** verify data 
                 */
      case 0x2F: /* Verify data on the media */
      case 0x55: /* Allow the host to set parameters in a peripheral */
      case 0x5A: /* Report parameters to the host */
      case 0xA8: /* Read (12) Transfer binary data from the media to the host */
      case 0xAA: /* Write (12) Transfer binary data from the host to the 
                 ** media 
                 */
      default:
         _process_unsupported_command(handle, ep_num, cbw_ptr);
         break;
   } /* Endswitch */

} /* EndBody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : service_ep1
* Returned Value : None
* Comments       :
*     Called upon a completed endpoint 1 (USB 1.1 Chapter 9) transfer
* 
*END*--------------------------------------------------------------------*/
static void service_ep1
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
      
      /* [IN] Is it a setup packet? */
      boolean              setup,
      
      /* [IN] Direction of the transfer.  Is it transmit? */
      uint_8               direction,
      
      /* [IN] Pointer to the data buffer */
      uint_8_ptr           buffer,
      
      /* [IN] Length of the transfer */
      uint_32              length      
   )
{ /* Body */
#if 1
   CBW_STRUCT_PTR cbw_ptr;
   
   if ((!direction) && (!CBW_PROCESSED)  && (length == 31)) {
      cbw_ptr = (CBW_STRUCT_PTR)((pointer)buffer);
   } /* Endif */
   
   if ((!direction) && (!CBW_PROCESSED) && (length == 31) && 
      (cbw_ptr->DCBWSIGNATURE == USB_DCBWSIGNATURE)) 
   {
      /* A valid CBW was received */
      csw.DCSWSIGNATURE = USB_DCSWSIGNATURE;
      csw.DCSWTAG = cbw_ptr->DCBWTAG;
      CBW_PROCESSED = TRUE;
      /* Process the command */
      _process_mass_storage_command(handle, BULKIN, cbw_ptr);
   } else {
      /* If a CBW was processed then send the status information and 
      ** queue another cbw receive request, else just queue another CBW receive
      ** request if we received an invalid CBW 
      */
      if (CBW_PROCESSED) {
         CBW_PROCESSED = FALSE;
         /* Send the command status information */
         _usb_device_send_data(handle, BULKIN, (uchar_ptr)&csw, 13);                
         _usb_device_recv_data(handle, BULKOUT, ep1_buf, 31);       
      }
      else if (!direction) {
         /* Invalid CBW received. Queue another receive buffer */                 
         _usb_device_recv_data(handle, BULKOUT, ep1_buf, 31);       
      } /* Endif */
      
   } /* Endif */

#else
	uint_8 data_send[16*16] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
								1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
								
								
	static FLAG = TRUE;
	
	if(direction == USB_SEND)
	{
		/* IN Transfer completed */
		FLAG = TRUE;
	
	}
	
	if(direction == USB_RECV && length > 0 && FLAG)
	{
		FLAG = FALSE;
		_usb_device_send_data(handle, BULKIN, (uchar_ptr)data_send,255);
	}
	
#endif

   return;
} /* Endbody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : service_speed
* Returned Value : None
* Comments       :
*     Called upon a speed detection event.
* 
*END*--------------------------------------------------------------------*/
static void service_speed
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
   
      /* [IN] Unused */
      boolean              setup,
   
      /* [IN] Unused */
      uint_8               direction,
   
      /* [IN] Unused */
      uint_8_ptr           buffer,
   
      /* [IN] Unused */
      uint_32              length      
   )
{ /* EndBody */
   speed = (uint_8)length;
   return;
} /* EndBody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : reset_ep0
* Returned Value : None
* Comments       :
*     Called upon a bus reset event.  Initialises the control endpoint.
* 
*END*--------------------------------------------------------------------*/
static void reset_ep0
   (
      /* [IN] Handle of the USB device */
      _usb_device_handle   handle,
   
      /* [IN] Unused */
      boolean              setup,
   
      /* [IN] Unused */
      uint_8               direction,
   
      /* [IN] Unused */
      uint_8_ptr           buffer,
   
      /* [IN] Unused */
      uint_32              length      
   )
{ /* Body */
   uint_32 status;

   status = _usb_device_recv_data(handle, 0, 0x0, 8);
   
   if (status != USB_OK)
   {
      return;
   }
   
   status = _usb_device_recv_data(handle, BULKOUT, ep1_buf, 31);
   
   if (status != USB_OK)
   {
      return;
   }
   
   return;
} /* EndBody */

/*FUNCTION*----------------------------------------------------------------
* 
* Function Name  : main
* Returned Value : None
* Comments       :
*     First function called.  Initialises the USB and registers Chapter 9
*     callback functions.
* 
*END*--------------------------------------------------------------------*/
void USB_dev_Task
   (
      uint_32 param
   )
{ /* Body */
   _usb_device_handle   handle;
   uint_8               error;

	_time_delay(10000);

   printf("\n\n-------------- Begin USB Disk Example --------------\n\n");
  
   _disable_interrupts();
   
    _int_install_unexpected_isr(); 
   _usb_driver_install(0, &_bsp_usb_callback_table);
  
   /* Initialize the USB interface */
   error = _usb_device_init(0, &handle, 2);
   
   if (error != USB_OK) {
      printf("\nUSB Initialization failed. Error: %x", error);
      fflush(stdout);
   } /* Endif */
   
   error = _usb_device_register_service(handle, USB_SERVICE_EP0, service_ep0);
   
   if (error != USB_OK) {
      printf("\nUSB Service Registration failed. Error: %x", error);
      fflush(stdout);
   } /* Endif */
   
   error = _usb_device_register_service(handle, USB_SERVICE_BUS_RESET, 
      reset_ep0);
   
   if (error != USB_OK) {
      printf("\nUSB Service Registration failed. Error: %x", error);
      fflush(stdout);
   } /* Endif */
   
   error = _usb_device_register_service(handle, USB_SERVICE_SPEED_DETECTION, 
      service_speed);

   if (error != USB_OK) {
      printf("\nUSB Service Registration failed. Error: %x", error);
      fflush(stdout);
   } /* Endif */
         
   error = _usb_device_register_service(handle, BULKOUT, service_ep1);
   
   if (error != USB_OK) {
      printf("\nUSB Service Registration failed. Error: %x", error);
      fflush(stdout);
   } /* Endif */

/* Temp */
   error = _usb_device_register_service(handle, BULKIN, service_ep1);
   
   if (error != USB_OK) {
      printf("\nUSB Service Registration failed. Error: %x", error);
      fflush(stdout);
   } /* Endif */
/* end */

#if 0
   MASS_STORAGE_DISK = (uchar_ptr)_mem_alloc_system_zero(TOTAL_LOGICAL_ADDRESS_BLOCKS*LENGTH_OF_EACH_LAB);
      
   if (MASS_STORAGE_DISK == NULL) {
      printf("\nDisk allocation failed");
      fflush(stdout);
   } /* Endif */
   
   memset(MASS_STORAGE_DISK, 0,
      (TOTAL_LOGICAL_ADDRESS_BLOCKS*LENGTH_OF_EACH_LAB));
   /* Format the "disk" */      
   memcpy(MASS_STORAGE_DISK, BOOT_SECTOR_AREA, 512);
   memcpy(MASS_STORAGE_DISK + 512, FAT16_SPECIAL_BYTES, 3);
   memcpy(MASS_STORAGE_DISK + 512*4, FAT16_SPECIAL_BYTES, 3);
#endif
   _enable_interrupts();
   printf ("\nUSB initialization complete.");
   printf ("\n\nAttach a USB cable between the host and device now.");
   printf ("\nThe device is now prepared for the enumeration process.");

   while (TRUE) {
      /* Do nothing */
   } /* Endwhile */
   
} /* Endbody */

/* EOF */