/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved
*
* Copyright (c) 1989-2008 ARC International;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: usbprv.h$
* $Version : 3.0.9.0$
* $Date    : Jul-20-2009$
*
* Comments:
*
*   This file contains the internal USB specific type definitions
*
*END************************************************************************/
#ifndef __usbprv_h__
#define __usbprv_h__ 1

#include "host_cnfg.h"

#define _EAI_DEBUG_  1

#ifndef __USB_OS_MQX__
#include "stdlib.h"
#include "string.h"

#define USB_lock()                           _disable_interrupts()
#define USB_unlock()                         _enable_interrupts()


/**********************************************************
The following lines have been commented because when MQX
is not used, the RTOS abstration layer provides the
implementation of USB_memalloc routine which aligns
the pointers on cache line size.
**********************************************************/
/*
#define USB_memalloc(n)                      malloc(((n) + (-(n) & 31)))
#define USB_memalloc(n)                      malloc(n)
#define USB_Uncached_memalloc(n)             malloc(n)

void * USB_memalloc(uint_32 n);
void  USB_memfree(void * aligned_ptr);

#define USB_memfree(ptr)                     free(ptr)
*/

#define USB_memzero(ptr,n)                   memset(ptr,0,n)
#define USB_memcopy(src,dst,n)               memcpy(dst,src,n)
#ifdef _DATA_CACHE_
	#define USB_dcache_invalidate()        		 _dcache_invalidate()
	#define USB_dcache_invalidate_line(p)        _dcache_invalidate_line(p)
	#define USB_dcache_invalidate_mlines(p,n)    _dcache_invalidate_mlines(p,n)
	#define USB_dcache_flush_line(p)        	 _dcache_flush_line(p)
	#define USB_dcache_flush_mlines(p,n)    	 _dcache_flush_mlines(p,n)
#else   // _DATA_CACHE_
	#define USB_dcache_invalidate()
	#define USB_dcache_invalidate_line(p)
	#define USB_dcache_invalidate_mlines(p,n)
	#define USB_dcache_flush_line(p)        
	#define USB_dcache_flush_mlines(p,n)

#endif


#define USB_install_isr(vector, isr, data)   USB_int_install_isr(vector, isr, data)

#else   // __USB_OS_MQX__

#include "mqx.h"
// jm #include "mcf5329.h"//This file contains the cache definitions. 
#include "psp.h"    // include platform specific configuration

#include "lwevent.h"

#define USB_lock()                           _int_disable()
#define USB_unlock()                         _int_enable()
//EAI
//#define USB_memalloc(n)                      _mem_alloc_system_zero(n)
//#define USB_Uncached_memalloc(n)			 _mem_alloc_system_zero(n)
//

#define USB_memalloc(n)                      USB_mem_alloc_system_zero(n)
#define USB_Uncached_memalloc(n)			 _mem_alloc_system_zero_uncached(n)
//#define USB_Uncached_memalloc(n)			 _mem_alloc_system_zero(n)
//#define USB_memfree(ptr)                     _mem_free(ptr)

//extern pointer USB_Uncached_memalloc(uchar_ptr f, uint_32 l, uint_32 n)			 ;
//extern uint_32 USB_memfree(uchar_ptr f, uint_32 l,pointer p);

#define USB_memzero(ptr,n)                   _mem_zero(ptr,n)
#define USB_memcopy(src,dst,n)               _mem_copy(src,dst,n)
#define USB_install_isr(vector, isr, data)   _int_install_isr(vector, isr, data)

#ifdef _DATA_CACHE_
#if 0
	#define USB_dcache_invalidate()        		 _DCACHE_INVALIDATE()
	#define USB_dcache_invalidate_line(p)        _DCACHE_INVALIDATE_LINE(p)
	/* Based on the targets it should be modified, for colfire it is MBYTES */
	#define USB_dcache_invalidate_mlines(p,n)    _DCACHE_INVALIDATE_MBYTES(p,n)
	#define USB_dcache_flush_line(p)        	 _DCACHE_FLUSH_LINE(p)
	/* Based on the targets it should be modified, for colfire it is MBYTES */
	#define USB_dcache_flush_mlines(p,n)    	 _DCACHE_INVALIDATE_MBYTES(p,n)
#else
	#define USB_dcache_invalidate()        		_mcf5329_cache_invalidate()
	#define USB_dcache_invalidate_line(p)        _mcf5329_cache_invalidate()
	/* Based on the targets it should be modified, for coldfire it is MBYTES */
	#define USB_dcache_invalidate_mlines(p,n)    _mcf5329_cache_invalidate()
	#define USB_dcache_flush_line(p)        	  
	/* Based on the targets it should be modified, for coldfire it is MBYTES */
	#define USB_dcache_flush_mlines(p,n)    	 
#endif
#else
	#define USB_dcache_invalidate()
	#define USB_dcache_invalidate_line(p)
	#define USB_dcache_invalidate_mlines(p,n)
	#define USB_dcache_flush_line(p)        
	#define USB_dcache_flush_mlines(p,n)
#endif

#endif


#endif

/* EOF */

