/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved
*
* Copyright (c) 2004-2008 Embedded Access Inc.;
* All Rights Reserved
*
* Copyright (c) 1989-2008 ARC International;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: flash_mcf51xx.c$
* $Version : 3.0.2.0$
* $Date    : May-22-2009$
*
* Comments:
*
*   The file contains functions for internal flash read, write, erase 
*
*END************************************************************************/

#include "mqx.h"
#include "bsp.h"
#include "fio.h"

#include "flashx.h"
#include "flash_mcf51xx.h"
   
/* Internal function definitions */
static void mcf51xx_ram_function(void); 
static void mcf51xx_ram_function_end (void);

boolean mcf51xx_prepare_ram_function(void);
static uchar_ptr ramcode_ptr = NULL;
static vuchar_ptr p_fstat_register;

/* allow write to flash */
const uchar MY_NVPROT  @0x0000040D = 0xFF;

/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    : _mcf51xx_set_cfmclkd
* Returned Value   : none
* Comments         :
*    Set the CFMCLKD register.
*
*END*----------------------------------------------------------------------*/
void _mcf51xx_set_cfmclkd()
{
	#define PRDIV8_LIMIT_FREQ	12800000L // limit freq. for using PRDIV8_PRSC
	#define FLASH_FREQ			200000L	  // 190 kHz program frequency
	#define PRDIV8_PRSC			8		  // prescaller 8
	
	VMCF51XX_FTSR_STRUCT_PTR  ftsr_ptr;
	
	ftsr_ptr = _bsp_get_cfm_address();
	
	if(BSP_BUS_CLOCK > PRDIV8_LIMIT_FREQ)
	{
		ftsr_ptr->FCDIV = MCF51XX_FTSR_FCDIV_PRDIV8 | MCF51XX_FTSR_FCDIV_FDIV((BSP_BUS_CLOCK+PRDIV8_PRSC*FLASH_FREQ-1) / (PRDIV8_PRSC * FLASH_FREQ) -1);
	}
	else
	{
		ftsr_ptr->FCDIV = MCF51XX_FTSR_FCDIV_FDIV((BSP_BUS_CLOCK+FLASH_FREQ-1) / FLASH_FREQ -1);
	}
}

/*FUNCTION*-------------------------------------------------------------------
* 
* Function Name    : mcf_51xx_flash_init
* Returned Value   : 
* Comments         :
*  	Erase a flash memory block  
*
*END*----------------------------------------------------------------------*/
void mcf_51xx_flash_init()
{
	VMCF51XX_FTSR_STRUCT_PTR  ftsr_ptr;
	
	/* get the pointer to cfm registers structure */
	ftsr_ptr = _bsp_get_cfm_address();
	p_fstat_register = &ftsr_ptr->FSTAT; 	
}

/*FUNCTION*-------------------------------------------------------------------
* 
* Function Name    : mcf51xx_flash_deinit
* Returned Value   : 
* Comments         :
*  	  
*
*END*----------------------------------------------------------------------*/
void mcf_51xx_flash_deinit()
{
	/* de-allocate the ram function buffer */
	if(ramcode_ptr){
		_mem_free(ramcode_ptr);
		ramcode_ptr = NULL;
	}
}	

/*FUNCTION*-------------------------------------------------------------------
* 
* Function Name    : mcf51xx_flash_deinit
* Returned Value   : 
* Comments         :
*  	  
*
*END*----------------------------------------------------------------------*/
boolean mcf51xx_prepare_ram_function()
{
	/* Test for ramcode_ptr memmory allocation */
	if(!ramcode_ptr)
	{		
	    /* Allocate space on stack to run flash command out of SRAM */
		ramcode_ptr = _mem_alloc((char*)mcf51xx_ram_function_end  - 
		    (char*)mcf51xx_ram_function);	
		/* copy code to RAM buffer */
		_mem_copy((void*)mcf51xx_ram_function, ramcode_ptr, 
		    (char*)mcf51xx_ram_function_end  - (char*)mcf51xx_ram_function);
	}	
	
	if(ramcode_ptr){
		_mem_set_type(ramcode_ptr,MEM_TYPE_IO_FLASHX_RAM_FUNCT);    
		return(TRUE);
	}
		
	else
		return(FALSE);
}

	
/*FUNCTION*-------------------------------------------------------------------
* 
* Function Name    : mcf51xx_erase_sector
* Returned Value   : true if success, or false
* Comments         :
*  	Erase a flash memory block  
*
*END*----------------------------------------------------------------------*/
boolean mcf51xx_erase_sector(IO_FLASHX_STRUCT_PTR flash_ptr, uchar_ptr from,
      _mem_size size)
{
    
    VMCF51XX_FTSR_STRUCT_PTR  ftsr_ptr;
	int (*RunInRAM)(void);
	
	/* allocate memory for ram function */
	if(!mcf51xx_prepare_ram_function())
		return(FALSE);
	
	*RunInRAM = (int(*)(void))ramcode_ptr;
	
	/* get the pointer to cfm registers structure */
	ftsr_ptr = _bsp_get_cfm_address();
	
	_int_disable();
	/* prepare flash write operation, clear flags and wait for ready state */
 	ftsr_ptr->FSTAT = (MCF51XX_FTSR_FSTAT_FPVIOL | MCF51XX_FTSR_FSTAT_FACCERR);
	while (!(ftsr_ptr->FSTAT & MCF51XX_FTSR_FSTAT_FCBEF)){
		/* wait */
	};

	/* latch address in Flash */
	(*(volatile vuint_32 *)(from)) = (vuint_32)-1;	
    
    /* issue erase command */
    ftsr_ptr->FCMD = MCF51XX_FTSR_FCMD_SECTOR_ERASE;
	
	/* run command and wait for it to finish (must execute from RAM) */	
	RunInRAM();	
	
  	if (ftsr_ptr->FSTAT & (MCF51XX_FTSR_FSTAT_FACCERR | MCF51XX_FTSR_FSTAT_FPVIOL)) {
  		_int_enable();
  		return FALSE;
  	}
  	  	
  	_int_enable();
	return(TRUE);	
         
}

/*FUNCTION*-------------------------------------------------------------------
* 
* Function Name    : mcf51xx_write_sector
* Returned Value   : true if success, or false
* Comments         :
*  	Performs a write into flash memory  
*
*END*----------------------------------------------------------------------*/
boolean mcf51xx_write_sector(IO_FLASHX_STRUCT_PTR flash_ptr, 
      uchar_ptr from_ptr, uchar_ptr to_ptr, _mem_size size) 
{

	#define BYTES_IN_WORD	4
	
	VMCF51XX_FTSR_STRUCT_PTR  ftsr_ptr;
	VMCF51XX_PMC_STRUCT_PTR   pmc_ptr;  
		
	int (*RunInRAM)(void);
	uint_32 temp_data = 0xFFFFFFFF;
	uchar_ptr temp_data_ptr;
	uchar byte_data_counter = 0;
	/* get the offset in write word */
	uint_32 offset = (uint_32)to_ptr & 0x00000003;;

	(uint_32_ptr)temp_data_ptr =  &temp_data;
	
	/* allocate memory for ram function */
	if(!mcf51xx_prepare_ram_function())
		return(FALSE);

	*RunInRAM = (int(*)(void))ramcode_ptr;

	/* get the pointer to cfm registers structure */
	ftsr_ptr = _bsp_get_cfm_address();
	/* get the pointer to pmc registers structure */
	pmc_ptr = _bsp_get_pmc_address();    
	/* Clear any errors */
	_int_disable();
	ftsr_ptr->FSTAT = (MCF51XX_FTSR_FSTAT_FPVIOL | MCF51XX_FTSR_FSTAT_FACCERR); 
	
	/* if the start address !=  doesn't correspond with hardware, prepare
	   variables for 1st word write */
	if(offset){
		/* Align pointer to writable address */
		to_ptr -= offset;
		/* jump over old data */
		byte_data_counter = offset;	
	}
	/* while are some data to write */
	while(size){
		/* move data to write word */		
		while( byte_data_counter < BYTES_IN_WORD && size ){
			*(temp_data_ptr+byte_data_counter) = *from_ptr++;
			byte_data_counter++;
			size--;
		}
		/* test the LVDF flag - if 1, we need to write data in 2 steps */
		if( pmc_ptr->SPMSC1 & MCF51XX_PMC_SPMSC1_LVDF) {
		    /* write odd bytes */
		    (*(vuint_32 *)(to_ptr)) = temp_data | 0x00FF00FF;
		    /* write command to CFMCMD */
		    ftsr_ptr->FCMD = MCF51XX_FTSR_FCMD_BURST_PROGRAM;
		    /* run command and wait for it to finish (must execute from RAM) */		
		    RunInRAM();
		    /* write even bytes */				
		    (*(vuint_32 *)(to_ptr)) = temp_data | 0xFF00FF00;
		    /* write command to CFMCMD */
		    ftsr_ptr->FCMD = MCF51XX_FTSR_FCMD_BURST_PROGRAM;
		    /* run command and wait for it to finish (must execute from RAM) */		
		    RunInRAM();
		} 
		else {  
    		/* move write data to register */
    		(*(vuint_32 *)(to_ptr)) = temp_data;
    		
    		/* write command to CFMCMD */
    		ftsr_ptr->FCMD = MCF51XX_FTSR_FCMD_BURST_PROGRAM;
    		/* run command and wait for it to finish (must execute from RAM) */		
    		RunInRAM();				
		}
		/* Check for Errors */
		if (ftsr_ptr->FSTAT & (MCF51XX_FTSR_FSTAT_FPVIOL | MCF51XX_FTSR_FSTAT_FACCERR)) {
			_int_enable();
			return (FALSE);
		}
		
		/* init variables for next loop */
		to_ptr += BYTES_IN_WORD;
		byte_data_counter = 0;
		temp_data = 0xFFFFFFFF;
	}
	_int_enable();
	return (TRUE);	
}

/*FUNCTION*-------------------------------------------------------------------
* 
* Function Name    : mcf51xx_erase_flash
* Returned Value   : true if success, or false
* Comments         :
*  	Erase a flash memory  
*
*END*----------------------------------------------------------------------*/
boolean mcf51xx_erase_flash(IO_FLASHX_STRUCT_PTR flash_ptr)
{
	int i, j;
	FLASHX_BLOCK_INFO_STRUCT_PTR   block_info_ptr;
	_mqx_uint sectors_num;
	_mqx_uint block_groups = flash_ptr->BLOCK_GROUPS;
	uchar_ptr erase_sector_ptr = flash_ptr->BASE_ADDR;
	
	block_info_ptr = flash_ptr->BLOCK_INFO_PTR;
	sectors_num = block_info_ptr->NUM_SECTORS;
	
	for(j = 0; j< flash_ptr->BLOCK_GROUPS; j++)
	{
		for(i = 0; i < sectors_num; i++)
		{
			if( mcf51xx_erase_sector( flash_ptr, erase_sector_ptr, 0 ) == FALSE)
			{
				return (FALSE);
			}
			erase_sector_ptr += block_info_ptr->SECT_SIZE;	
		}
	}
	return(TRUE);		
}

/********************************************************************/
/*
 *	Code required to run in SRAM to perform flash commands. 
 *  All else can be run in flash.
 *	In p_reg we need address of FSTAT register.
 *
 ********************************************************************/ 
static void mcf51xx_ram_function() 
{
	asm(
		/* Saves registers on stack */
		lea     -24(a7),a7
        movem.l d1-d2/d5-d7/a4,(a7)
	
		move.w  SR,D7          	/* current sr            */
		move.l  D7,D5           /* save return value     */
		move.w  #0x0700,D6      /* set level to 7        */
		andi.l  #0x0000F8FF,D7  /* zero out current IPL  */
		or.l    D6,D7           /*   */
		move.w  D7,SR           /* place new IPL in sr   */

		// FSTAT = CBEIF;
		
		move.l   p_fstat_register,a4
		move.b   #0x80,d1     
		move.b   d1,(a4)

		// while (!(FSTAT & MCF51XX_FTSR_FSTAT_FCBEF)){};
		//wait until execution complete
		loop:
		move.l   p_fstat_register,a4
		move.b   (a4),d2
		moveq    #0,d1
		move.b   d2,d1
		andi.l   #0x40,d1     
		beq.s    loop

		move.w  D5,SR
		
		/* Restores the registers from stack */
        movem.l  (a7),d1-d2/d5-d7/a4
        lea      24(a7),a7
	);
}

static void mcf51xx_ram_function_end (void){} 
