/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved
*
* Copyright (c) 2004-2008 Embedded Access Inc.;
* All Rights Reserved
*
* Copyright (c) 1989-2008 ARC International;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: flash_mcf52223.c$
* $Version : 3.0.2.0$
* $Date    : May-21-2009$
*
* Comments:
*
*   The file contains functions for internal flash read, write, erase 
*
*END************************************************************************/
#include "mqx.h"
#include "bsp.h"
 
#include "flash_mcf52xx.h"
#include "flash_mcf52223.h"

/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    : _bsp_get_cfm_address
* Returned Value   : Address upon success
* Comments         :
*    This function returns the base register address of the CFM
*
*END*----------------------------------------------------------------------*/
pointer _bsp_get_cfm_address()
{
   return (pointer)(&((VMCF5222_STRUCT_PTR)_PSP_GET_IPSBAR())->CFM); 
}


/*FUNCTION*-------------------------------------------------------------------
*
* Function Name    : _mcf52223_internal_flash_install
* Returned Value   : uint_32 a task error code or MQX_OK
* Comments         :
*    Install a internal flash device.
*
*END*----------------------------------------------------------------------*/
uint_32 _mcf52223_internal_flash_install(
	/* [IN] A string that identifies the device for fopen */
    char_ptr           identifier,
    uint_32            flashx_size
) 
{ /* Body */
	static FLASHX_BLOCK_INFO_STRUCT _mcf52223_flashx_block_map[2];
	FLASHX_INIT_STRUCT init_structure;

	/* prepare flash sector/block map based on LCF data */
	_mcf52223_flashx_block_map[0].NUM_SECTORS = flashx_size / FLASHX_SECT_SIZE;
	_mcf52223_flashx_block_map[0].START_ADDR = 0;
	_mcf52223_flashx_block_map[0].SECT_SIZE =  FLASHX_SECT_SIZE;
	_mcf52223_flashx_block_map[1].NUM_SECTORS = 0;
	_mcf52223_flashx_block_map[1].START_ADDR = 0;
	_mcf52223_flashx_block_map[1].SECT_SIZE = 0;

	if(!flashx_size || (FLASHX_END_ADDR - FLASHX_START_ADDR) < flashx_size )
		return FLASHXERR_INVALID_SIZE;	
 	
	/* set the timer prescaller for flash opperations */
    _mcf52xx_set_cfmclkd();
	
	/* prepare flashx initialization structure */
	init_structure.ID_PTR = identifier;
	init_structure.SECTOR_ERASE = (boolean (_CODE_PTR_)(IO_FLASHX_STRUCT_PTR, uchar_ptr,
      _mem_size))mcf52xx_erase_sector; 
    init_structure.SECTOR_PROGRAM = (boolean (_CODE_PTR_)(IO_FLASHX_STRUCT_PTR, 
      uchar_ptr, uchar_ptr, _mem_size))mcf52xx_write_sector;
    init_structure.CHIP_ERASE = (boolean(_CODE_PTR_)(IO_FLASHX_STRUCT_PTR))mcf52xx_erase_flash;  
	init_structure.INIT = (void (_CODE_PTR_))mcf_52xx_flash_init;
	init_structure.DEINIT = (void (_CODE_PTR_))mcf_52xx_flash_deinit;
	init_structure.WRITE_PROTECT = NULL;
	init_structure.MAP_PTR = _mcf52223_flashx_block_map;
	init_structure.BASE_ADDR = (void *)FLASHX_START_ADDR; // defined in lcf file
	init_structure.WIDTH = 32;
	init_structure.DEVICES = 1;
	init_structure.WRITE_VERIFY = 0;
	
	return _io_flashx_install(&init_structure);
	
} /* Endbody */
