/*
 *	ROMCopy.c		-	Routine to copy from ROM to RAM.
 *
 *	Copyright  1993-1998 Metrowerks, Inc. All Rights Reserved.
 *	Copyright  2005 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 */

#include "ROMCopy.h"

#include <string.h>

#pragma PID off

#pragma define_section init ".init" far_absolute R

/*
 *	Simple byte copy routine ...
 */
__declspec(init) far __asm void simple_block_copy(void *dst:__A1, const void *src:__A0, unsigned long size:__D0)
{
__loop_start__:
	move.b		(a0)+,(a1)+

	subq.l		#1,d0
	bne			__loop_start__

	rts
}

/*
 *	Routine to copy a single section from ROM to RAM ...
 */
__declspec(init) far void __copy_rom_section(void* dst, const void* src, unsigned long size)
{
	if (size && (dst != src))
		simple_block_copy(dst, src, size);
}

/*
 *	Routine that copies all sections the user marked as ROM into
 *	their target RAM addresses ...
 *
 *	__S_romp is automatically generated by the linker if it
 *	is referenced by the program.  It is a table of RomInfo
 *	structures.  The final entry in the table has all-zero
 *	fields.
 */
__declspec(init) far void __copy_rom_sections_to_ram(void)
{
	int				index;

	/*
	 *	Go through the entire table, copying sections from ROM to RAM.
	 */
	for (index = 0;
		 _S_romp[index].Source != NULL ||
		 _S_romp[index].Target != NULL ||
		 _S_romp[index].Size != 0;
		 ++index)
	{
		__copy_rom_section( (void *)_S_romp[index].Target,
							(const void *)_S_romp[index].Source,
							_S_romp[index].Size );
							
	} 	
}
